# ===========================================================================
# Crypt::GCrypt:Sexp
#
# Perl interface to multi-precision integers from the GNU Cryptographic library
#
# Author: Michael Schroeder E<lt>mls@suse.deE<gt>,
# Copyright © 2022.
#
# Use this software AT YOUR OWN RISK.
# See below for documentation.
#

package Crypt::GCrypt::Sexp;

use strict;
use warnings;

use Crypt::GCrypt;

1;
__END__

=encoding utf8

=head1 NAME

Crypt::GCrypt::Sexp - Perl interface to S-expressions from the GNU Cryptographic library

=head1 SYNOPSIS

  use Crypt::GCrypt::Sexp;

  my $sexp = Crypt::GCrypt::Sexp->new('(foo)');

=head1 ABSTRACT

Crypt::GCrypt::Sexp provides an object interface to S-expressions
from the C libgcrypt library.

=head1 BASIC OPERATIONS

=head2 new($buffer)

Create a new S-expression object from its external representation

  my $sexp = Crypt::GCrypt::Sexp->new('(foo)');

The argument is expected to be in canonized format. To autodetect
the format, add 1 as second parameter.

=head2 build($template, $arg1, ...)

Build a S-expression from a template and arguments. See the
libgcrypt documentation for the formatting characters supported
in the template.

  my $sexp = Crypt::GCrypt::Sexp->build('(p %M)', $mpi);

=head2 find_token($token)

Scan the S-expression for a a sublist with type token. Return
a new S-expression consisting of the sublist or undef when
not found.

  my $subsexp = $sexp->find_token('p');

=head2 nth_mpi($number, $mpifmt)

Return the mpi at the position $number in the S-expression.

  my $mpi = $sexp->nth_mpi(1);

It is possible to specify a token string as additional parameter
to select the nth mpi of the sublist matching the token:

  my $mpi_p = $sexp->nth_mpi(1, Crypt::GCrypt::MPI::FMT_USG, 'p');

=head2 nth_data($number)

Return the data at the position $number in the S-expression.
You can optionally specify a token like with nth_mpi.

  my $value = $sexp->nth_data(1, 'value')

=head2 nth($number)

Return a new S-expression from the element at positon $numer.
You can optionally specify a token like with nth_mpi.

  my $subsexp = $sexp->nth(1);

=head1 OUTPUT AND DEBUGGING

=head2 dump()

Send the Sexp to the libgcrypt debugging stream.

=head2 print($format)

Return a string with the data of this S-expression, in a given format.  See
CONVERSION FORMATS for the available formats.

=head1 CONVERSION FORMATS

The available printing and scanning formats are all in the
Crypt::GCrypt::Sexp namespace, and have the same meanings as in gcrypt.

=head2 FMT_DEFAULT

A convenient external S-expression representation

=head2 FMT_CANON

The canonical format of a S-expression

=head2 FMT_BASE64

Not currently supported in libgcrypt

=head2 FMT_ADVANCED

An advanced representation of the S-expression

=head1 AUTHORS AND CONTRIBUTORS

Michael Schroeder E<lt>mls@suse.deE<gt>

=head1 COPYRIGHT AND LICENSE

Copyright © Michael Schroeder
Crypt::GCrypt::Sexp is free software, you may redistribute it and/or
modify it under the same terms as Perl itself.

=head1 DISCLAIMER

This software is provided by the copyright holders and contributors ``as
is'' and any express or implied warranties, including, but not limited to,
the implied warranties of merchantability and fitness for a particular
purpose are disclaimed. In no event shall the regents or contributors be
liable for any direct, indirect, incidental, special, exemplary, or
consequential damages (including, but not limited to, procurement of
substitute goods or services; loss of use, data, or profits; or business
interruption) however caused and on any theory of liability, whether in
contract, strict liability, or tort (including negligence or otherwise)
arising in any way out of the use of this software, even if advised of the
possibility of such damage.

=cut
